#! /bin/env python
# unit tests for cert module
#  $Id: test_cert.py,v 1.12 2006/05/19 23:50:35 mwalfish Exp $

#
# Copyright (C) 2005-2006 Michael Walfish
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
import dqm.cryptkey_wrap.cryptkey as cryptkey
import dqm.crypt.cert as dqm_cert
import dqm.err as dqm_err
import datetime
import sys

if __name__ == "__main__":

    priv_authority = cryptkey.gen_rsa_privkey(2048,3)
    pub_authority  = priv_authority.to_pubkey()

    priv_supplicant = cryptkey.gen_rsa_privkey(1024,3)
    pub_supplicant  = priv_supplicant.to_pubkey()

    signer_id = "test_authority_id"
    quota     = 3056
    expiries  = [datetime.date(7000,01,01), datetime.date(2004,01,01)]

    DELIM = dqm_cert.SimpleCert.DELIM
    cm = dqm_cert.SimpleCert()

    def helper_func(expiry):
	cert_str = cm.create(signer_id, pub_supplicant, quota, expiry, 
			       priv_authority)
	return cm.verify(cert_str, pub_authority) 

    def good_faith(priv_key):
	return cm.create(signer_id, pub_supplicant, quota, expiries[0],
			 priv_key)

    def check(cert_str):
	return cm.verify(cert_str, pub_authority)

    [rc1, rc2] = map(helper_func, expiries)

    cert_str3 = DELIM.join([signer_id, 
			    "-234:234", 
			    str(quota),
			    expiries[0].isoformat(), 
			    "1234341234124"])
    cert_str8 = DELIM.join([signer_id, 
			    "-234*234", 
			    str(quota),
			    expiries[0].isoformat(), 
			    "1234341234124"])

    cert_str4 = "134adsasd213094sfj;dsfkcxz0932	4;lsafdkj;lfdsj"

    cert_str5 = good_faith(priv_authority)
    twiddled_char = cert_str5[-10:-9]
    if (twiddled_char == "A"): twiddled_char = "B"
    else: twiddled_char = "A"
    cert_str5 = cert_str5[:-10] + twiddled_char + cert_str5[-9:]
    
    # this is the wrong private key; the signature should fail
    cert_str6 = good_faith(priv_supplicant)
    [rc3, rc4, rc5, rc6, rc8] = map(check, 
			    [cert_str3, cert_str4, cert_str5, cert_str6,
			     cert_str8])

    cert = cm.str_to_cert(cert_str6)
    # reconstructing sig (remember, this is for the wrong key, but that's
    # okay because we're not testing verification), just well-formedness
    sig  = priv_supplicant.sign((DELIM.join(cert_str6.split(DELIM)[:-1])))

    def weakkey_eq(key1,key2):
        return key1.to_str() == key2.to_str()

    rc7 = (cert['signer_id'] == signer_id      and
	   cert['quota']     == quota          and
	   cert['expiry']    == expiries[0]    and
	   cert['sig']       == sig            and 
	   weakkey_eq(cert['pub_key'],pub_supplicant))

    tests = [(lambda: rc1 == dqm_err.SUCCESS, 
			     "simple check?"),
	     (lambda: rc2 == dqm_err.EXPIRED_CERT,
			     "bogus expiry fails?"),
	     (lambda: rc3 == dqm_err.WRONG_FORMAT, 
	                     "recognizes too many fields?"),
	     (lambda: rc4 == dqm_err.WRONG_FORMAT,
			     "recognizes an ill-formed cert?"),
	     (lambda: rc5 == dqm_err.INVALID_CERT_SIG,
			     "recognizes hacked signature?"),
	     (lambda: rc6 == dqm_err.INVALID_CERT_SIG,
			     "recognizes signature of wrong key?"),
	     (lambda: rc7, "str_to_cert passes sanity?"),
	     (lambda: rc8 == dqm_err.WRONG_FORMAT, 
			     "recognizes ill-formed public key?")]
	    
    for test,msg in tests:
	print msg,
	if not test(): print "FAILED"; sys.exit(1)
	else: print "passed"

    sys.exit(0)

