#
# $Id: wrap_gen_dbpool.sh,v 1.4 2006/03/15 18:10:27 mwalfish Exp $
#
# Create a new disposable pool and rewrite the config file. The point is
# to make the creation of disposables atomic. We're trying to handle the
# case when an e-mail is sent while the pool of disposables is being
# regenerated.
#
if [ $# != 3 ]
then
    echo "usage: $0 config-file db-gen quota"
    exit 1
fi

config_file=$1
db_gen=$2
quota=$3
if [[ ! -f $config_file ]]
then
    echo "couldn't find file $config_file"
    exit 1
fi

if [[ ! -f $db_gen ]]
then
    echo "couldn't find stamp generator $db_gen"
    exit 1
fi


entry='stamp_pool'

# that '^' character matters since stamp_pool is a substring of
# metastamp_pool [this comment is now obsolete.]
old_pool_name=`grep ^$entry $config_file | awk '{print \$2}'`

# get the name part.
file_name=`basename $old_pool_name`
old_pool_dir=${old_pool_name%%/$file_name}
# strip off the last part of the dir_name

base_dir_name=${old_pool_dir%.*}
num=${old_pool_dir##*.}
num=$(( $num + 1 ))  # nasty shell syntax for ++num

new_pool_name=$base_dir_name.$num/$file_name

# generate the new quota
$db_gen $quota $new_pool_name

# edit the config file to point to a new place. this edit operation is
# hopefully atomic. see the comment above about the '^' character
perl -p -i -e "s#^$entry\s.*\$#$entry $new_pool_name#" $config_file

# give time for any current users of the old files to finish up their
# business
sleep 10
rm -rf $old_pool_dir

