#! /usr/bin/env python 
#
# Simple stand-alone client of the portal to test RPC calls.
# 
#
#
#
# Copyright (C) 2005-2006 Michael Walfish
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
import sys
import dqm.rpc.rpc_udp as rpc_udp
import dqm.user.process_res as process_res
import dqm.svc.dqm_portal as dqm_portal
import dqm.rpc.events as dqm_events
import dqm.user.dqm_client as dqm_client
import sha
from dqm.util.convert import *

# stupid. should be a way to get these from dqm_types
codes = { "DQM_OK" : 0,
          "DQM_NOT_FOUND" : 1,
          "DQM_BAD_METASTAMP" : 2,
          "DQM_METASTAMP_REUSED" : 4,
          "DQM_SEEN_KEY" : 8,
          "DQM_NO_MORE_STORAGE" : 16,
          "DQM_BAD_PARAM" : 32 }
strcodes = {}

for k in codes.keys():
    strcodes[codes[k]] = k

def check(rc,res):
    if rc == rpc_udp.CLI_TIMED_OUT:
	print "RPC timed out"
    elif rc == rpc_udp.CLI_SENDING_ERROR:
	print "RPC sending error: %s" % str(res)
    elif rc == rpc_udp.CLI_CALL_COMPLETED:
	pass
    else: 
	assert(False)

    return rc == rpc_udp.CLI_CALL_COMPLETED

def set_req (caller_func, key, value):

    ms = "this is a dummy metastamp"
    arg = dqm_client.pack_set_args(key, value, ms)

    caller_func(dqm_portal.DQM_PORTAL_PROC_SET, arg, set_resp)

def set_resp (dummy, ret):
    rc, res = ret
    if check(rc,res):
	if res is None: print "We got denied (set_resp)"
	else: print strcodes[res]
    dqm_events.stop_looping()

def test_req (caller_func, key):
    arg = key
    caller_func(dqm_portal.DQM_PORTAL_PROC_TEST, arg, test_resp, key)

def test_resp (orig_key, ret):
    rc, res_h = ret
    if check(rc,res_h):
	if res_h is None: print "We got denied (test_resp)"
	else: 
	    res = process_res.process_res(res_h)
	    if res.status == 0 and sha.new(res.val).digest() != orig_key:
		print "BAD_HASH_VALUE"
	    else:
		print strcodes[res.status]	
    dqm_events.stop_looping()

if __name__ == "__main__":

    def usage():
	print """usage: %s host:port val test|set
	    "<host:port>" is the host and port of the RPC server
	    "val" will be sha-1'ed to arrive at the key""" % sys.argv[0]
    def usage_and_exit():
	usage()
	sys.exit(2)

    argc = len(sys.argv)
    if argc != 4: usage_and_exit()

    op = sys.argv[3]
    if op not in ['test', 'set']: usage_and_exit()

    # store user's arguments
    try:
	host,port = sys.argv[1].split(":")
	port = int(port)
	value = sha.new(sys.argv[2]).digest()
	key = sha.new(value).digest()

    except ValueError:
	usage_and_exit()

    rpc_cli = rpc_udp.rpc_client()

    # this next bit is confusing (sorry.)
    # all it says is: if user requested a test, do a test.
    # if user requested a set, do a set.
    def rpc_caller(proc, arg, cb, cb_arg=None):
	rpc_cli.call(dqm_portal,
	             dqm_portal.DQM_PORTAL_PROGRAM,
		     dqm_portal.DQM_PORTAL_VERSION,
		     proc, arg,
		     (host, port), 
		     10.0, cb, cb_arg)

    ops = {"test" :  lambda : test_req(rpc_caller, key),
           "set"  : lambda : set_req(rpc_caller, key, value) }

    print "Will try to do %s for:" % op 
    print "key:\t%s" % to_hex(key)
    if op == "set":
	print "value:\t%s" % to_hex(value)
    ops[op]()

    dqm_events.loop()
