#! /bin/env python
#
# $Id: stamp-verifier.py,v 1.4 2006/05/19 23:50:35 mwalfish Exp $
#
#  Reads a set of e-mail headers on stdin and attaches an X-dqm-checked 
#  header, the value of which is given by whether there was a stamp in the
#  original header, whether the stamp verifies, and whether the stamp was
#  previously in the DQM. This script is specific to mail handling and
#  thus should probably moved out of the DQM-in-general directory to the
#  mail-specific directory.
#
#
# Copyright (C) 2005-2006 Michael Walfish
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
import sys
import dqm.config.user as dqm_config
import dqm.user.rcvr as dqm_rcvr
import dqm.err as dqm_err
import email
import email.Errors
import email.Parser
import dqm.logger

if __name__ == "__main__":
    """This is a simple command-line app to verify a stamp that is
    read from stdin."""

    def usage():
	print """usage: %s config-file [log-file]""" % sys.argv[0]

    def usage_and_exit():
	usage()
	sys.exit(3)

    if len(sys.argv) not in [2, 3]:
	usage_and_exit()

    if len(sys.argv) == 2: logger = dqm.logger.stderr_logger()
    else:                  logger = dqm.logger.file_logger(sys.argv[2])

    config_obj = dqm_config.parse_config(sys.argv[1])

    from_stdin = sys.stdin.read()

    try: 

	parser = email.Parser.Parser()
	# the 'True' means 'parse only the headers'
	msg = parser.parsestr(from_stdin, True)

    except email.Errors.MessageParseError: 
	logger.log('Ill-formed message: ' + from_stdin)
	sys.exit(1)

    stamp_str = msg.get('X-dqm-stamp')

    if stamp_str is None:
	logger.log('No stamp on message')
	# XXX: better log message?
	rc = dqm_err.STAMPLESS_EMAIL
    else:
	# next line gets rid of whitespace in the stamp
	stamp_str = "".join(stamp_str.split())
	Rcvr = dqm_rcvr.Receiver(config_obj, logger)
	rc = Rcvr.process_stamp(stamp_str)

    msg.add_header('X-dqm-checked', dqm_err.str_codes[rc])
   
    # Print out the envelope header and then just the headers. 
    # If we print out the whole message, we get dummy MIME 'next-part's 
    # when the message header indicates that the message is MIME.
    if (msg.get_unixfrom() is not None):
	print msg.get_unixfrom()
    for hdr,val in msg.items():
	print hdr + ': ' + val
    # don't forget the newline!
    print 
