/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <time.h>
#include <signal.h>

void usage(int argc, char *argv[]) {
  printf("usage: %s [-s sample time (ms)] [-t run time (s)] port1 port2 ...\n",
	 argv[0]);
  printf(" port1 port2 ...: <= 100 ports\n");
  exit(1);
}

int index(unsigned short n, unsigned short *h, int len) {
  for (int i = 0; i < len; ++i)
    if (n == h[i])
      return i;
  return -1;
}

int samples;
unsigned short *ports;
int *sums;
int numports;

inline double millis(struct timeval *tv) {
  return ((double) tv->tv_sec + 
	  ((double) tv->tv_sec / 1000000000.0)) * 1000;
}

void sighandler(int sig) {
  for (int i = 0; i < numports; ++i) {
    printf("port %d: avg: %f\n", ports[i], (float) sums[i] / (float) samples);
  }
  exit(0);
}
  

int main(int argc, char *argv[]) {

  ports = (unsigned short *) calloc(100, sizeof(unsigned short));
  sums = (int *) calloc(100, sizeof(int));
  numports = 0;

  FILE *f = fopen("/proc/net/udp", "r");

  long sampletime = 100; // in ms
  long runtime = 30; // in s

  for (int i = 1; i < argc; ++i) { // ~no error-checking, in the name of time
    const char *arg = argv[i];
    if (arg[0] == '-') {
      switch (arg[1]) {
      case 's':
	sampletime = atol(argv[++i]);
	if (sampletime < 0)
	  usage(argc,argv);
	break;
      case 't':
	runtime = atol(argv[++i]);
	if (runtime < 0)
	  usage(argc,argv);
	break;
      default:
	usage(argc,argv);
      }
    } else {
      ports[numports++] = (short) atol(arg);
    }
  }

  printf("Looking for ports: ");
  for (int i = 0; i < numports; ++i)
    printf("%d ", ports[i]);

  printf("\n");

  struct timespec sleeptime;

  if (sampletime > 999) {
    sleeptime.tv_sec = sampletime / 1000;
    sampletime = sampletime % 1000;
    sleeptime.tv_nsec = sampletime * 1000000;
  } else {
    sleeptime.tv_sec = 0;
    sleeptime.tv_nsec = sampletime * 1000000;
  }

#define LINESIZE 128 
  char buf[LINESIZE+1];
  size_t nread;

#define PORTLOC 15
  unsigned short port = 0;
#define QLENLOC 46
  long qlen = 0;
  
  int ind;

  struct timeval starttime;
  gettimeofday(&starttime, NULL);
  //double begintime = millis(&starttime);
  struct timeval curtime;
  gettimeofday(&curtime, NULL);
  int lastprint = 0;

  samples = 0;

  signal(SIGINT, sighandler);

  while (1) {
    samples++;
    rewind(f);
    fread(buf, sizeof(char), LINESIZE, f); // kill the first (header) line
    while ((nread = fread(buf, sizeof(char), LINESIZE, f)) > 0) {
      buf[nread] = 0;
      port = (unsigned short) strtol(buf + PORTLOC, NULL, 16);
      if ((ind = index(port, ports, numports)) >= 0) {
	qlen = strtol(buf + QLENLOC, NULL, 16);
        sums[ind] += qlen;
      }
    }
    if ((samples - lastprint) > (1.0 / ((float) sampletime / 1000)) * 5) {
      double lasttime = millis(&curtime);
      gettimeofday(&curtime, NULL);
      double nowtime = millis(&curtime);
      
      printf("Rate: %f\n", 
	     (double) (samples - lastprint) * 1000 / (nowtime - lasttime));
      lastprint = samples;
      if (curtime.tv_sec - starttime.tv_sec > runtime)
	sighandler(2);
    }    
    nanosleep(&sleeptime, NULL);
  }
}
