/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#include "lru_cache.h"

void
lru_cache::dump()
{
    warnx("queue:\n");

    for (kv_rec* curr = q.first; curr; curr = q.next(curr)) {
	warnx << "key: " 
	      << hexdump(curr->key->base(), curr->key->size())
	      << " val: "
	      << hexdump(curr->val->base(), curr->val->size())
	      << "\n";
    }
}

kv_rec*
lru_cache::lookup(ptr<dqm_key> key, bool do_touch)
{
    kv_rec* ret = map[key];
    if (!ret) return NULL;

    if (do_touch)
	touch(ret);

    return ret;
}


void
lru_cache::remove(kv_rec* rec)
{
    q.remove(rec);
    map.remove(rec);
    curr_size--;
    delete rec;
}

void
lru_cache::add(ptr<dqm_key> k, ptr<dqm_val> v)
{ 
    kv_rec* rec = New kv_rec(k,v);
    if (!add(rec))
	delete rec;	
}

bool
lru_cache::add(kv_rec* rec)
{
    kv_rec* dummy;
    if ((dummy = lookup(rec->key, true)) != NULL) 
	return false;

    if (curr_size == max_size) {
	// evict 
	kv_rec* front = q.first;	
	remove(front);
    }

    ++curr_size;
    map.insert(rec);
    q.insert_tail(rec);
    return true;
}


void
lru_cache::touch(kv_rec* rec)
{
    q.remove(rec);
    q.insert_tail(rec);
}
