/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#ifndef _PORTAL_H_
#define _PORTAL_H_

#include "async.h"
#include "arpc.h"
#include "dqm_node.h"
#include "nodetable.h"

class svccb;
class asrv;
class aclnt;
class storer;
class base_logger;
class lru_cache;

void stats_printer_wrapper();
void stats_printer();

class portal : public virtual refcount {

    public:
	portal(const approved_list*, 
	       u_int16_t portal_port,
	       ptr<aclnt>, 
	       ptr<nodetable>, 
	       ptr<lru_cache>,
	       ptr<storer>,
	       node_id,
	       ptr<base_logger>,
	       bool* exit_token);

	~portal();

	void dispatch(svccb*);

	void stop();

    private:

	void handle_NULL(svccb*);
	void handle_SET(svccb*);
	void handle_TEST(svccb*);
	void local_get_tmo(ptr<bool> called, svccb* rpc, ptr<dqm_key> key);
	void local_get_cb(ptr<bool> called, timecb_t* tmo, 
	                    svccb* rpc, ptr<dqm_key> key, ptr<dqm_val> val);
        void handle_TEST_2(svccb* rpc, ptr<dqm_key> key, ptr<dqm_val> loc_val);


	void complete_TEST(svccb*, ptr<dqm_key> key, 
	                   bool found, ptr<dqmstorer_get_res>);
	void complete_SET(svccb*, u_int32_t);

	void init_sockaddrs(const approved_list*);

	void stats_printer_wrapper();
	void stats_printer();

	ptr<asrv>        srv;
	ptr<nodetable>   nodetbl;
	ptr<lru_cache>   cache;
	ptr<storer>      local_storer;
	node_id          local_id;
	ptr<aclnt>       clnt;
	ptr<base_logger> logger;
	bool*            exit_token;
};

typedef callback<void, bool, ptr<dqmstorer_get_res> >::ref getter_cb;
typedef callback<void, u_int32_t>::ref putter_cb;

// The 'getter' and the 'putter' are helper classes for the Portal.
//
// The getter does the dirty work of doing GETs at a sequence of nodes.
// When it's done, it calls back getter_cb with two arguments: (rc,val).
// rc is true if the key was found and false otherwise. If rc is true,
// val is a ptr to the dqmstorer_get_res object that contains the value
// to be returned.
class getter {

    public:
	getter(ptr<dqm_key>, ptr<node_list>, getter_cb,
	       ptr<storer> local_storer, node_id, ptr<aclnt> clnt,
	       ptr<base_logger> logger);
	~getter();


    private:
	void make_get_req();
	void make_get_req_worker();
	void get_cb(ptr<dqmstorer_get_res>, clnt_stat err);
	bool process_rpc_reply(ptr<dqmstorer_get_res> res, clnt_stat err);
	void iterate();

	ptr<dqm_key>     key;
	getter_cb        cb;
	ptr<node_list>   path;
	u_int32_t        pos;
	ptr<storer>      local_storer;
	node_id          local_id;
	ptr<aclnt>       clnt;
	ptr<base_logger> logger;
};

class putter {

    public:
	putter(ptr<dqm_key> key, ptr<dqm_val> val,
	       ptr<node_list> path, putter_cb cb, 
	       ptr<storer> local_storer,
	       node_id     local_id,
	       ptr<aclnt> clnt,
	       ptr<base_logger> logger);
	~putter();

    private:
	void make_put_req();
	void make_put_req_worker();
	void put_cb(dqm_stat*, clnt_stat err);
	void process_rpc_reply(dqm_stat res_copy, clnt_stat err);
	void iterate();

	ptr<dqm_key>   key;
	ptr<dqm_val>   val;
	ptr<node_list> path;
	u_int32_t      pos;
	putter_cb      cb;
	u_int32_t      errs;
	ptr<storer>    local_storer;
	node_id        local_id;
	ptr<aclnt>      clnt;
	ptr<base_logger> logger;
};

#endif
