/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include "async.h"
#include "nodetable.h"
#include "nodetable_geom.h"
#include "nodetable_notree.h"
#include "sha1.h"

#define NUM_MACHINES   500

int main(int argc, char** argv)
{
    int num_machines;

    if (argc > 2) {
	printf("usage: %s [num_nodes]\n", argv[0]);
	exit(1);
    } else if (argc == 2) {
	num_machines = atoi(argv[1]);
    } else {
	num_machines = NUM_MACHINES;
    }

    warnx("number of machines: %d\n", num_machines);

    char pre_hash [200];
    char keybuf [sha1::hashsize];
    int i;
    u_int32_t j;
    nodetable_notree the_table(.05);
//    nodetable_geom the_table(2.5, 50);
    nodetable* nodetbl = &the_table;
    dqm_key k;
    ptr<node_list> path;

    ptr<dqm_key> key = New refcounted<dqm_key>();

    for (i = 0; i < num_machines; i++) {

	strbuf bunker_name_strbuf("my_qwerty_name%d", i);
	str bunker_name(bunker_name_strbuf);

	ip_port my_ip_port;
	my_ip_port.ip   = htonl(i);
	my_ip_port.port = htons(i);

	nodetbl->add_node(my_ip_port, bunker_name);

	if (i == 26)
	    nodetbl->set_local_node(my_ip_port);

	nodetbl->set_state(my_ip_port, UP);
    }

//    nodetbl->dump();

    for (i = 0; i < 10 && i < num_machines; i++) {

	sprintf(pre_hash, "%d", i);
	sha1_hash(keybuf, pre_hash, strlen(pre_hash));
	strncpy(key->base(), keybuf, sha1::hashsize);

	path = nodetbl->calc_path_get(key);
#ifdef PRINT_PUT_PATH
	ptr<node_list> path2 = nodetbl->calc_path_put(key);
#endif

	warnx << hexdump(key->base(), key->size()) << "\n";

	for (j = 0; j < path->size(); j++) {
	    warnx << (*path)[j] << "\n";
	}
#ifdef PRINT_PUT_PATH
	warnx << "put path\n";
	for (j = 0; j < path2->size(); j++) {
	    warnx << (*path2)[j] << "\n";
	}
#endif
	warnx << "\n";
    }

    for (i = num_machines; i > 0; i--) {

	ip_port my_ip_port;
	my_ip_port.ip = htonl(i-1);
	my_ip_port.port = htons(i-1);
	nodetbl->remove_node(my_ip_port);
    }

    return 0;
}
